==> lmtpd-0.9.19.tar.gz: lmtpd-0.9.19/re/re-pcre.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] = 
"$Id: re-pcre.c,v 1.9 2003/11/27 01:13:31 lwa Exp $";

#include "config.h"

#include <sys/types.h>
#include <stdlib.h>
#include <string.h>
#include "re.h"

#if WITH_PCRE

#include <pcre.h>

static char pcre_malloc_error;
static void *(*orig_pcre_malloc)(size_t);

void *my_pcre_malloc(size_t l) {
  void *m;
  m = (orig_pcre_malloc)(l);
  pcre_malloc_error = (m==NULL);
  return m;
}


static void re_close(RE *re) {
  if (re->code) {
    (pcre_free)(re->code);
  }
  if (re->hint) {
    (pcre_free)(re->hint);
  }
}

static int re_compile(RE *re, char *pattern, int icase) {
  int erroffset;
  orig_pcre_malloc = pcre_malloc;
  pcre_malloc = my_pcre_malloc;

  re->code = pcre_compile(pattern, re->options|(icase?PCRE_CASELESS:0),
			  &re->errstr, &erroffset, NULL);
  if (re->code == NULL) {
    pcre_malloc = orig_pcre_malloc;
    if (pcre_malloc_error) {
      re->err = RE_ERR_MEMORY;
    } else {
      re->err = RE_ERR_SYNTAX;
    }
    return re->err;
  }
  re->hint = pcre_study(re->code, 0, &re->errstr);

  pcre_malloc = orig_pcre_malloc;

  if (re->errstr != NULL) {
    (pcre_free)(re->code);
    re->code = NULL;
    if (pcre_malloc_error) {
      re->err = RE_ERR_MEMORY;
    } else {
      re->err = RE_ERR_SYNTAX;
    }
    return re->err;
  }
  return 0;
}

static int re_exec(RE *re, char *string) {
  int ret = pcre_exec(re->code, re->hint, string, strlen(string)+1,
		      0, 0, NULL, 0);
  return ret!=0;
}

RE re_pcre = {
  re_compile,
  re_exec,
  re_close,
  0,
  0,
};

#ifdef PCRE_UTF8
RE re_pcre_utf8 = {
  re_compile,
  re_exec,
  re_close,
  PCRE_UTF8,
  0,
};
#else
extern RE re_error;
RE re_pcre_utf8 = re_error;
#endif

#endif

==> lmtpd-0.10.7.tar.gz: lmtpd-0.10.7/re/re-pcre.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#if WITH_PCRE

static char const rcsid[] UNUSED = 
"$Id: re-pcre.c,v 1.11 2004/10/27 10:42:08 lwa Exp $";

#include <sys/types.h>
#include <stdlib.h>
#include <string.h>
#include "re.h"


#include <pcre.h>

static char pcre_malloc_error;
static void *(*orig_pcre_malloc)(size_t);

void *my_pcre_malloc(size_t l) {
  void *m;
  m = (orig_pcre_malloc)(l);
  pcre_malloc_error = (m==NULL);
  return m;
}


static void re_close(RE *re) {
  if (re->code) {
    (pcre_free)(re->code);
  }
  if (re->hint) {
    (pcre_free)(re->hint);
  }
}

static int re_compile(RE *re, char *pattern, int icase) {
  int erroffset;
  orig_pcre_malloc = pcre_malloc;
  pcre_malloc = my_pcre_malloc;

  re->code = pcre_compile(pattern, re->options|(icase?PCRE_CASELESS:0),
			  &re->errstr, &erroffset, NULL);
  if (re->code == NULL) {
    pcre_malloc = orig_pcre_malloc;
    if (pcre_malloc_error) {
      re->err = RE_ERR_MEMORY;
    } else {
      re->err = RE_ERR_SYNTAX;
    }
    return re->err;
  }
  re->hint = pcre_study(re->code, 0, &re->errstr);

  pcre_malloc = orig_pcre_malloc;

  if (re->errstr != NULL) {
    (pcre_free)(re->code);
    re->code = NULL;
    if (pcre_malloc_error) {
      re->err = RE_ERR_MEMORY;
    } else {
      re->err = RE_ERR_SYNTAX;
    }
    return re->err;
  }
  return 0;
}

static int re_exec(RE *re, char *string) {
  int ret = pcre_exec(re->code, re->hint, string, strlen(string)+1,
		      0, 0, NULL, 0);
  return ret!=0;
}

RE re_pcre = {
  re_compile,
  re_exec,
  re_close,
  0,
  0,
};

#ifdef PCRE_UTF8
RE re_pcre_utf8 = {
  re_compile,
  re_exec,
  re_close,
  PCRE_UTF8,
  0,
};
#else
extern RE re_error;
RE re_pcre_utf8 = re_error;
#endif

#endif
